/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework;



import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestSuite;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour;



/**
 * This type describes the test harness.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 12 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class TestHarness
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER          = LoggerFactory.getLogger(TestHarness.class);
    
    
    private final List<ITestBehaviour>    testBehaviours  = new ArrayList<ITestBehaviour>();
    
    private Set<Class<?>>                 classes         = new HashSet<Class<?>>();
    
    private Set<Object>                   objects         = new HashSet<Object>();
    
    private int                           failedInstances = 0;
    
    
    private int                           failedClasses   = 0;
    
    
    
    /**
     * Builds a test harness.
     */
    public TestHarness() {
    
    
        super();
    }
    
    
    /**
     * This method performs the addition of a new behaviour to process on the class to test.
     * 
     * @param _newInstance
     *            the new instance.
     */
    public void addBehaviour(final ITestBehaviour _newInstance) {
    
    
        if (_newInstance == null) {
            throw new IllegalArgumentException();
        }
        this.testBehaviours.add(_newInstance);
    }
    
    
    /**
     * This method performs the processing of all tests.
     * 
     * @return the processing of all tests.
     */
    public boolean processTests() {
    
    
        boolean res = true;
        LOGGER.debug("Initialization of the tests");
        final List<SuiteInstanceTest> testingInstances = this.testingInstances();
        final List<SuiteClassTest> testingClasses = this.testingClasses();
        
        for (final SuiteInstanceTest suite : testingInstances) {
            final boolean individualRes = suite.testingInstanceObject();
            if (!individualRes) {
                LOGGER.error("\t ->[FAILED]");
                this.failedInstances++;
            }
            res &= individualRes;
            
        }
        for (final SuiteClassTest suite : testingClasses) {
            final boolean individualRes = suite.testingClass();
            if (!individualRes) {
                LOGGER.error("\t ->[FAILED]");
                this.failedClasses++;
            }
            res &= individualRes;
        }
        
        LOGGER.info("Tests finished", this.failedInstances, this.failedClasses);
        LOGGER.info("\tInstance failed {} on {}", this.failedInstances, this.objects.size());
        LOGGER.info("\tClasses failed {} on {}", this.failedClasses, this.classes.size());
        return res;
        
    }
    
    
    /**
     * This method provides a bank of tests for junit
     * 
     * @param _suite
     *            the suite
     */
    public void provideTests(final TestSuite _suite) {
    
    
        for (final SuiteClassTest test : this.testingClasses()) {
            _suite.addTest(test);
        }
        for (final SuiteInstanceTest test : this.testingInstances()) {
            _suite.addTest(test);
        }
    }
    
    
    /**
     * This method removes a test.
     * 
     * @param _class
     *            the class
     */
    public void removeTest(final Class<? extends ITestBehaviour> _class) {
    
    
        for (final ITestBehaviour behaviour : this.testBehaviours) {
            if (_class.equals(behaviour.getClass())) {
                this.testBehaviours.remove(behaviour);
                break;
            }
        }
    }
    
    
    /**
     * This method sets the classes to analysis.
     * 
     * @param _setClasses
     *            the classes to analysis.
     */
    public void setClasses(final Set<Class<?>> _setClasses) {
    
    
        if (_setClasses == null) {
            throw new IllegalArgumentException();
        }
        
        this.classes = _setClasses;
    }
    
    
    /**
     * This method sets the classes to analysis.
     * 
     * @param _setObjects
     *            the classes to analysis.
     */
    public void setObjects(final Set<Object> _setObjects) {
    
    
        if (_setObjects == null) {
            throw new IllegalArgumentException();
        }
        this.objects = _setObjects;
        
        
    }
    
    
    /**
     * This method performs the testing classes.
     * 
     * @return the result of tests
     */
    private List<SuiteClassTest> testingClasses() {
    
    
        final List<SuiteClassTest> res = new ArrayList<SuiteClassTest>(this.classes.size());
        LOGGER.debug("{} classes to test; ", this.classes.size());
        for (final Class<?> clazz : this.classes) {
            res.add(new SuiteClassTest(clazz, this.testBehaviours));
        }
        return res;
        
    }
    
    
    /**
     * This method performs the testing classes.
     * 
     * @return the result of tests
     */
    private List<SuiteInstanceTest> testingInstances() {
    
    
        final List<SuiteInstanceTest> res = new ArrayList<SuiteInstanceTest>(this.objects.size());
        LOGGER.debug("{} instances to test; ", this.objects.size());
        for (final Object object : this.objects) {
            res.add(new SuiteInstanceTest(this.testBehaviours, object));
        }
        return res;
    }
    
    
}
